@@ -16,7 +16,7 @@ MODULE Calcs_K
 CONTAINS
 
   
-  SUBROUTINE findKmax(i, kmax, tmax, mmax, mfirst, left_Of_Max)
+  SUBROUTINE findKmax(i, kmax, tmax, mmax, mfirst, left_Of_Max, error)
     ! Finds the value of Kmax, Tmax, and Mmax.
     ! Also return the first value of m (mfirst) amd whether this is to the left of the max (left_Of_Max).
   
@@ -24,17 +24,18 @@ CONTAINS
     USE ISO_C_BINDING, ONLY: C_INT, C_DOUBLE, C_BOOL
     USE Calcs_Imag
     USE Calcs_Solvers
+    
     IMPLICIT NONE
     
     REAL(KIND=C_DOUBLE), INTENT(OUT)    :: kmax, tmax
     INTEGER(C_INT), INTENT(OUT)         :: mmax, mfirst
     LOGICAL(C_BOOL), INTENT(OUT)        :: left_Of_Max
     INTEGER(C_INT), INTENT(IN)          :: i
+    LOGICAL(C_BOOL)                     :: error
   
-    REAL(KIND=C_DOUBLE)     :: pi, t_Start_Point, slope_At_Zero, Imk_value
+    REAL(KIND=C_DOUBLE)     :: t_Start_Point, slope_At_Zero, Imk_value
     REAL(KIND=C_DOUBLE)     :: aimrerr, tmaxL, tmaxR, ratio, threshold, t_small
-    LOGICAL(C_BOOL)         :: error
-    
+    LOGICAL(C_BOOL)         :: errorHere
     
     ! Grab the relevant scalar values for this iteration:
     current_y    = Cy(i)    ! Access y value for index i
@@ -50,21 +51,18 @@ CONTAINS
     !       a. If p near 1, approximate starting point by p=1 case analytically
     !       b. Otherwise, use standard starting point
   
-  
-    ! Grab the relevant scalar values for this iteration:
-    current_y    = Cy(i)    ! Access y value for index i
-    current_mu   = Cmu(i)   ! Access mu value for index i
-    current_phi  = Cphi(i)  ! Access phi value for index i
-    
     
     ! --- Initialization ---
     aimrerr = 1.0E-09_C_DOUBLE
-    pi = 4.0_C_DOUBLE * DATAN(1.0_C_DOUBLE)
     threshold = 1.0E5_C_DOUBLE ! Threshold for a "large" tmax
   
   
     ! Find starting points
-    CALL evaluateImkd(0.0_C_DOUBLE, slope_At_Zero)
+    CALL evaluateImkd(0.0_C_DOUBLE, slope_At_Zero, errorHere)
+    IF (errorHere) THEN
+      error = .TRUE.
+      IF (Cverbose) CALL DBLEPR("ERROR: Evaluating Imkd: t", -1, slope_At_Zero, 1)
+    END IF
 
     IF (slope_At_Zero .LE. 0.0_C_DOUBLE) THEN
       ! A. Im k(t) initially heads DOWNWARDS
@@ -102,9 +100,12 @@ CONTAINS
   
         tmax = threshold
         t_Start_Point = tInitialGuess()   ! This computes large-t approx
-        CALL evaluateImk(tmax, kmax, error)    ! Now find the corresponding value of kmax
-        IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, t_Start_Point, 1)
-  
+        CALL evaluateImk(tmax, kmax, errorHere)    ! Now find the corresponding value of kmax
+        IF (errorHere) THEN
+          error = .TRUE.
+          IF (Cverbose) CALL DBLEPR("ERROR: Evaluating Imk: t", -1, tmax, 1)
+        END IF
+
       ELSE
         ! Compute ratio
         ratio = current_y / current_mu
@@ -126,22 +127,20 @@ CONTAINS
                 IF (t_Start_Point <= 0.0_C_DOUBLE) t_Start_Point = 1.0E-12_C_DOUBLE
             END IF
         END IF
-!  WRITE(*,*) "GOT START PT:", t_Start_Point
         ! Now find kmax and tmax using this t_Start_Point
   !      IF (Cpsmall) THEN
           tmaxL = 0.0_C_DOUBLE       ! Since Left bound can be zero
           tmaxR = t_Start_Point * 2.0_C_DOUBLE
-!  WRITE(*,*) "  BOUNDS 1: RTSAFE", tmaxL, tmaxR
-          CALL improveKmaxSPBounds(t_Start_Point, tmaxL, tmaxR)
-!  WRITE(*,*) "  BOUNDS 1 revises: RTSAFE", tmaxL, tmaxR
+
+          CALL improveKmaxSPBounds(t_Start_Point, tmaxL, tmaxR, error)
+
           ! Crudely improve the bounds that bracket the starting point for finding Kmax.
-          CALL rtsafe(  evaluateImkdZero,   &
-                        tmaxL,              &
-                        tmaxR,              &
-                        aimrerr,            &
-                        tmax,               &
-                        error)
-          IF (error) CALL DBLEPR("ERROR: cannot solve", -1, tmax, 1)
+          CALL rtsafe(  evaluateImkdZero, tmaxL, tmaxR, aimrerr, tmax, errorHere)
+          IF (errorHere) THEN
+            error = .TRUE.
+            IF (Cverbose) CALL DBLEPR("ERROR: rtsafe: tmax", -1, tmax, 1)
+          END IF
+
       END IF  
   
       ! Find mmax, which depends on whether we are working with the PDF or the CDF.
@@ -151,12 +150,16 @@ CONTAINS
   
       ! Find kmax from tmax
       CALL evaluateImk(tmax, kmax, error)
-      IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, tmax, 1)
+      IF (errorHere) THEN
+        error = .TRUE.
+        IF (Cverbose) CALL DBLEPR("ERROR: Evaluating Imk: t", -1, tmax, 1)
+      END IF
+
       ! Find mmax from kmax
       IF (Cpdf) THEN
-        mmax = FLOOR(2.0_C_DOUBLE * kmax / pi) - 1
+        mmax = FLOOR(2.0_C_DOUBLE * kmax / PI) - 1
       ELSE
-        mmax = FLOOR(kmax / pi)
+        mmax = FLOOR(kmax / PI)
       END IF
 
       ! Establish the first value of m to use, and whether the first zero is to the left of kmax
@@ -183,11 +186,10 @@ CONTAINS
       
         IMPLICIT NONE
     
-        REAL(KIND=C_DOUBLE)     :: t0, pi
+        REAL(KIND=C_DOUBLE)     :: t0
         REAL(KIND=C_DOUBLE)     :: ratio, r, t_small, t_large, c, angle
       
-        pi = 4.0_C_DOUBLE * DATAN(1.0_C_DOUBLE)
-      
+
         ! Weighting exponent for interpolation
         r = 4.0E0_C_DOUBLE ! Artibtrary
       
@@ -198,18 +200,15 @@ CONTAINS
         IF (ratio < 1.0E0_C_DOUBLE) THEN
            t_small = (current_mu**(1.0E0_C_DOUBLE - Cp) / current_phi) *      &
                       DSQRT( (2.0E0_C_DOUBLE/Cp) * (1.0E0_C_DOUBLE - ratio) )
-  !WRITE(*,*) "ratio=", ratio
-  !WRITE(*,*) "entering small-t branch"
-  
         ELSE
            ! If y >= mu, no small-t root: set small approx to zero
            t_small = 0.0E0_C_DOUBLE
         ENDIF
       
         ! --- Large-t approximation ---
-        angle   = 0.5D0 * pi / (Cp - 1.0E0_C_DOUBLE)
+        angle   = 0.5D0 * PI / (Cp - 1.0E0_C_DOUBLE)
         c       = DCOS(angle)**(Cp - 1.0E0_C_DOUBLE)
-       t_large = ( current_mu**(2.0_C_DOUBLE*(1.0_C_DOUBLE - Cp)) /   &
+        t_large = ( current_mu**(2.0_C_DOUBLE*(1.0_C_DOUBLE - Cp)) /  &
                   ((1.0_C_DOUBLE - Cp) * current_phi) ) *             &
                   ( current_y**(Cp - 1.0_C_DOUBLE) )
       
@@ -217,9 +216,12 @@ CONTAINS
         t0 = ratio**r * t_small + (1.0E0_C_DOUBLE - ratio**r) * t_large
       
       END FUNCTION tInitialGuess
+
   
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
+
       
-      SUBROUTINE improveKmaxSPBounds(startTKmax, tmaxL, tmaxR)
+      SUBROUTINE improveKmaxSPBounds(startTKmax, tmaxL, tmaxR, error)
         ! Crudely improve the bounds that bracket the starting point for finding Kmax.
         ! Sometime a decent starting point is crucial for timely convergence.
         USE ISO_C_BINDING, ONLY: C_INT, C_DOUBLE
@@ -242,9 +244,19 @@ CONTAINS
         boundL = tmaxL
         boundR = tmaxR
   
-        CALL evaluateImkd(boundL, slopeL)
-        CALL evaluateImk(boundL, Imk_value, error)
-        IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, boundL, 1)
+        CALL evaluateImkd(boundL, slopeL, errorHere)
+        IF (errorHere) THEN
+          error = .TRUE.
+          IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundL, 1)
+          RETURN
+        END IF
+
+        CALL evaluateImk(boundL, Imk_value, errorHere)
+        IF (errorHere) THEN
+          error = .TRUE.
+          IF (Cverbose) CALL DBLEPR("ERROR: derivative zero =", -1, boundL, 1)
+          RETURN
+        END IF
         
         ! Slope at starting point should always be positive if we are here:
         ! if the slope is negative, Im k(t) heads down and m = -1, -2, ...
@@ -254,7 +266,7 @@ CONTAINS
         DO WHILE (keep_Searching)
           oldBoundL = boundL
           boundL = (boundL + 1.0E-2_C_DOUBLE) * 1.250E0_C_DOUBLE
-          CALL evaluateImkd(boundL, slopeL)
+          CALL evaluateImkd(boundL, slopeL, error)
           CALL evaluateImk(boundL, Imk_value, error)
           IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, boundL, 1)
   
@@ -274,11 +286,21 @@ CONTAINS
         IF (tmaxL .GT. boundR) boundR = tmaxL * 2.0_C_DOUBLE 
         
         ! Find the slope at boundR the starting point (SP)
-        CALL evaluateImkd(boundR, slopeR)
+        CALL evaluateImkd(boundR, slopeR, errorHere)
+        IF (errorHere) THEN
+          error = .TRUE.
+          IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+          RETURN          
+        END IF
         
         ! Find the value if kmax at the starting point (SP)
-        CALL evaluateImk(boundR, Imk_value, error)
-        IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+        CALL evaluateImk(boundR, Imk_value, errorHere)
+        IF (errorHere) THEN
+          error = .TRUE.
+          IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+          RETURN          
+        END IF
+        
   
         ! A valid upper bound:
         ! - must have a negative SLOPE (must be negative)i.e., heading down), *AND*
@@ -296,10 +318,20 @@ CONTAINS
             search_Its = search_Its + 1
             boundR = (boundR + 0.1_C_DOUBLE) * 2.0E0_C_DOUBLE
       
-            CALL evaluateImkd(boundR, slopeR)
-            CALL evaluateImk(boundR, Imk_value, error)
-            IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
-  
+            CALL evaluateImkd(boundR, slopeR, errorHere)
+            IF (errorHere) THEN
+              error = .TRUE.
+              IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+              RETURN          
+            END IF
+
+            CALL evaluateImk(boundR, Imk_value, errorHere)
+            IF (errorHere) THEN
+              error = .TRUE.
+              IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+              RETURN          
+            END IF
+
             IF ( (slopeR .LT. 0.0E0_C_DOUBLE ) .AND.  &
                  (Imk_value .GT. 0.0_C_DOUBLE) ) THEN
               ! - Found an upper bound where the slope is negative, and kmax is positive
@@ -314,10 +346,21 @@ CONTAINS
         DO WHILE (keep_Searching)
           oldBoundR = boundR
           boundR = boundR * 0.90E0_C_DOUBLE
-          CALL evaluateImkd(boundR, slopeR)
-          CALL evaluateImk(boundR, Imk_value, error)
-          IF (error) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
-  
+          CALL evaluateImkd(boundR, slopeR, errorHere)
+          IF (errorHere) THEN
+            error = .TRUE.
+            IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+            RETURN          
+          END IF
+
+          CALL evaluateImk(boundR, Imk_value, errorHere)
+          IF (errorHere) THEN
+            error = .TRUE.
+            IF (Cverbose) CALL DBLEPR("ERROR: integrand zero =", -1, boundR, 1)
+            RETURN          
+          END IF
+
+
           IF ( (slopeR .GT. 0.0E0_C_DOUBLE) .AND.   &
                (Imk_value .GT. 0.0_C_DOUBLE) ) THEN
             ! - Gone too far, so keep previous bound
@@ -332,9 +375,9 @@ CONTAINS
   
   END SUBROUTINE findKmax
   
-  
-  
-  
+
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
+
     
   REAL(KIND=C_DOUBLE) FUNCTION findKmaxSP() 
     ! Find the starting point for finding Kmax
@@ -346,13 +389,13 @@ CONTAINS
     IMPLICIT NONE
     
     REAL(KIND=C_DOUBLE)           :: tsmall, tlarge, abs1mp
-    REAL(KIND=C_DOUBLE)           :: omegaInf, slope, pi
-  
+    REAL(KIND=C_DOUBLE)           :: omegaInf, slope
+    LOGICAL(C_BOOL)               :: error
 
     ! Initialize
     abs1mp = ABS(1.0_C_DOUBLE - Cp)
-    pi = 4.0_C_DOUBLE * DATAN(1.0_C_DOUBLE)
-  
+    findKmaxSP = 0.0_C_DOUBLE
+    
     ! We find a small-t approx, a large-t approx, and a combined approx.
     ! The SP should be the FIRST of these whose slope is NEGATIVE
   
@@ -365,13 +408,14 @@ CONTAINS
       findKmaxSP = tsmall
     ELSE
       ! This should be the CASE: p > 2, AND y < mu
-      omegaInf = (pi / 2.0_C_DOUBLE) * &
+      omegaInf = (PI / 2.0_C_DOUBLE) * &
                  (1.0_C_DOUBLE - Cp)/(2.0_C_DOUBLE*Cp - 1.0_C_DOUBLE)
       tsmall = current_mu**(1.0_C_DOUBLE - Cp) / ( (1.0_C_DOUBLE - Cp)) * &
                DTAN(omegaInf)
       
-      CALL evaluateImkd(tsmall, slope)
-      
+      CALL evaluateImkd(tsmall, slope, error)
+      IF (error) RETURN
+    
       IF (slope .LE. 0.0_C_DOUBLE) THEN
         findKmaxSP = tsmall
         RETURN
@@ -382,7 +426,8 @@ CONTAINS
     tlarge = (current_mu / current_y)**(Cp - 1.0_C_DOUBLE) * &
               current_mu**(1.0_C_DOUBLE - Cp) / (current_phi * abs1mp)
   
-    CALL evaluateImkd(tlarge, slope)
+    CALL evaluateImkd(tlarge, slope, error)
+    IF (error) RETURN
   
     IF (slope .LE. 0.0_C_DOUBLE) THEN
       findKmaxSP = tlarge
@@ -395,9 +440,10 @@ CONTAINS
   END FUNCTION findKmaxSP
   
   
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!  
   
   
-  SUBROUTINE improveKZeroBounds(m, left_Of_Max, zeroMid, zeroL, zeroR)
+  SUBROUTINE improveKZeroBounds(m, left_Of_Max, zeroMid, zeroL, zeroR, error)
     ! Improve the bounds that bracket the zero of Im k(t).
     ! A decent starting point is sometimes crucial to timely convergence.
     
@@ -411,6 +457,7 @@ CONTAINS
     INTEGER(C_INT), INTENT(IN)          :: m
     REAL(KIND=C_DOUBLE), INTENT(INOUT)  :: zeroL, zeroR
     LOGICAL(C_BOOL), INTENT(IN)         :: left_Of_Max
+    LOGICAL(C_BOOL), INTENT(INOUT)      :: error
   
     REAL(KIND=C_DOUBLE)     :: valueL, valueR, multiplier
     REAL(KIND=C_DOUBLE)     :: df, valueMid
@@ -421,7 +468,11 @@ CONTAINS
 
     ! Initialisation
     maxSearch = 10  ! Don't spend too long, so set limit
-  
+    valueL = 0.0_C_DOUBLE
+    valueR = 0.0_C_DOUBLE
+    df = 0.0_C_DOUBLE
+    valueMid = 0.95_C_DOUBLE
+    
     ! Set multipier: this adjust the sign depending on whether we are
     ! left of the max (so left bound is negative) or to the right of
     ! the max (so left bound is positive)
@@ -439,11 +490,10 @@ CONTAINS
   
     ! FIND the function value of the starting point (SP)
     zeroMid = (zeroL + zeroR) / 2.0_C_DOUBLE
-    CALL evaluateImkM(zeroMid, valueMid, df, m)
-    CALL evaluateImkM(zeroL, valueL, df, m)
-    CALL evaluateImkM(zeroR, valueR, df, m)
-  
-  
+    CALL evaluateImkM(zeroMid, valueMid, df, m, error)
+    CALL evaluateImkM(zeroL,   valueL,   df, m, error)
+    CALL evaluateImkM(zeroR,   valueR,   df, m, error)
+
     ! CHECK IF BOUNDS REALLY DO BOUND THE ZERO:
     IF ( (valueL * valueR) .GE. 0.0_C_DOUBLE) THEN
       ! Bounds DO NOT trap the zero
@@ -458,7 +508,7 @@ CONTAINS
           ! We are on the LEFT of the maximum of Im k(t), but the L bound gives a +ive value.
           ! So we need to go LEFT a little.
           zeroL = (zeroL - 0.1_C_DOUBLE) * 0.95_C_DOUBLE
-          CALL evaluateImkM(zeroL, valueL, df, m)
+          CALL evaluateImkM(zeroL, valueL, df, m, error)
           
           IF (its .GE. 100_C_INT) stopIterating = .TRUE.
           IF( valueL .LE. 0.0_C_DOUBLE) stopIterating = .TRUE.
@@ -474,18 +524,14 @@ CONTAINS
           ! We are on the LEFT of the maximum of Im k(t), but the R bound gives a -ive value.
           ! So we need to go RIGHT a little.
           zeroR = (zeroR + 0.1_C_DOUBLE) * 1.05_C_DOUBLE
-          CALL evaluateImkM(zeroR, valueR, df, m)
+          CALL evaluateImkM(zeroR, valueR, df, m, error)
 
           IF (its .GE. 100_C_INT) stopIterating = .TRUE.
           IF( valueR .GE. 0.0_C_DOUBLE) stopIterating = .TRUE.
         END DO
         
       END IF
-  
-  
-  
-  
-  
+
       ! The solution depends on what side of the max we are.
       ! If to the RIGHT of the max of Im k(t), zeroL should give a +ive value; zeroR a -ive value.
       IF ( .NOT.(left_Of_Max) ) THEN
@@ -493,24 +539,19 @@ CONTAINS
           ! We are on the RIGHT of the maximum of Im k(t), but the L bound gives a -ive value.
           ! So we need to go LEFT a little.
           zeroL = (zeroL - 0.1_C_DOUBLE) * 0.95_C_DOUBLE
-          CALL evaluateImkM(zeroL, valueL, df, m)
+          CALL evaluateImkM(zeroL, valueL, df, m, error)
         END DO
-  
-  
-  
+
         ! The solution depends on what side of the max we are.
         ! If to the RIGHT of the max of Im k(t), zeroL should give a -ive value; zeroR a +ive value.
         DO WHILE ( valueR .GT. 0.0_C_DOUBLE) 
           ! We are on the RIGHT of the maximum of Im k(t), but the R bound gives a +ive value.
           ! So we need to go RIGHT a little.
           zeroR = (zeroR + 0.1_C_DOUBLE) * 1.05_C_DOUBLE
-          CALL evaluateImkM(zeroR, valueR, df, m)
+          CALL evaluateImkM(zeroR, valueR, df, m, error)
         END DO
       END IF
-  
-  
-  
-  
+
     ELSE
       ! Bounds DO trap the zero, so improve a little
     
@@ -524,10 +565,10 @@ CONTAINS
         
       ! And once more ONLY
       zeroMid = (zeroL + zeroR) / 2.0_C_DOUBLE
-      CALL evaluateImkM(zeroMid, valueMid, df, m)
-      CALL evaluateImkM(zeroL, valueL, df, m)
-      CALL evaluateImkM(zeroR, valueR, df, m)
-  
+      CALL evaluateImkM(zeroMid, valueMid, df, m, error)
+      CALL evaluateImkM(zeroL, valueL, df, m, error)
+      CALL evaluateImkM(zeroR, valueR, df, m, error)
+
       ! Find a point halfway between bounds.
       ! If the new point has same sign as L/R bound, make that the new L/R bounds.
       IF ( (valueMid * valueL) .GE. 0.0_C_DOUBLE) THEN
@@ -545,7 +586,7 @@ CONTAINS
   END SUBROUTINE improveKZeroBounds
   
   
-  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   
   
   SUBROUTINE advanceM(m, mmax, mOld, left_Of_Max)
@@ -595,89 +636,138 @@ CONTAINS
           left_Of_Max = .FALSE.
         END IF
       END IF
-!WRITE(*,*) " - Moving m from ", mOld, 'to', m
+
     END SUBROUTINE advanceM
     
     
-  !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     
     
-    SUBROUTINE findExactZeros(m, tL, tR, tStart, tZero, left_Of_Max) 
+    SUBROUTINE findExactZeros(m, tL, tR, tStart, tZero, left_Of_Max, error) 
   ! Find the exact zeros of the integrand
   
-  USE tweedie_params_mod
-  USE ISO_C_BINDING, ONLY: C_INT, C_DOUBLE, C_BOOL
-  USE Calcs_Solvers
-  USE Calcs_Imag
+    USE tweedie_params_mod
+    USE ISO_C_BINDING, ONLY: C_INT, C_DOUBLE, C_BOOL
+    USE Calcs_Solvers
+    USE Calcs_Imag
+    
+    IMPLICIT NONE
   
-  IMPLICIT NONE
-
-  INTEGER(C_INT), INTENT(IN)          :: m
-  REAL(KIND=C_DOUBLE), INTENT(INOUT)  :: tL, tR, tStart
-  REAL(KIND=C_DOUBLE), INTENT(OUT)    :: tZero
-  LOGICAL(C_BOOL), INTENT(IN)         :: left_Of_Max
-
-  REAL(KIND=C_DOUBLE)   :: xacc, fL, fR, dfL, dfR, tstart_update, tMid
-  LOGICAL(C_BOOL)       :: error
-
-  ! Sync the local value of  m  to the shared value.
-  m_shared = m
-  ! Set the accuracy
-  xacc = 1.0E-11_C_DOUBLE
-
-  ! Find mmax, which depends on whether we are working with the PDF or the CDF.
-  ! The PDF uses cos Im k(t) in the integrand; the CDF has sin Im k(t) in the integrand.
-  ! Thus, the PDF has integrand zeros at Im k(t) = pi/2 + m * pi/y;
-  !       the CDF has integrand zeros at Im k(t) =        m * pi/y.
-  ! Ensure the bounds actually bound the zero
-!WRITE(*,*)">>> IN findExactZeros: m, tL, tR", i, m, tL, tR
-  CALL evaluateImkM(tL, fL, dfL, m)
-!WRITE(*,*) "fL, dfL", fL, dfL
-  CALL evaluateImkM(tR, fR, dfR, m)
-!WRITE(*,*) "fR, dfR", fR, dfR
-  IF ( (fL * fR) .GT. 0.0_C_DOUBLE ) THEN
-    ! Then bounds do not bound the zero
-     tMid = (tL + tR) / 2.0_C_DOUBLE
- 
-    CALL improveKZeroBounds(m, left_Of_Max, tMid, tL, tR)
-    IF (Cverbose) CALL DBLEPR("Bounds do not bracket the zero (findExactZeros)", -1, fR, 1)
-  END IF
-  ! For robustness, use rtsafe when the  distance between zeros 
-  ! is expected to be small (i.e., in the tail).
-  IF ( m .LE. -3 ) THEN ! Use rtsafe whenever m is large and negative
-    CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, error)
-    IF (error) CALL DBLEPR("ERROR: cannot solve", -1, tZero, 1)
-  ELSE IF ( (Cpsmall) .AND. (current_y .LT. current_mu) ) THEN
-    ! When small p and small y, fight harder for good starting bounds
-    CALL improveKZeroBounds(m, left_Of_Max, tStart, tL, tR)
-    CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, error)
-    IF (error) CALL DBLEPR("ERROR: cannot solve", -1, tZero, 1)
-  ELSE
-    ! Default to rtnewton for "easy" cases (e.g., initial zeros)
-    tstart_update = (tL + tR) / 2.0_C_DOUBLE
-!    CALL rtnewton(i, evaluateImkM_wrapper, tstart_update, xacc, tZero)
-    CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, error)
-  END IF
+    INTEGER(C_INT), INTENT(IN)          :: m
+    REAL(KIND=C_DOUBLE), INTENT(INOUT)  :: tL, tR, tStart
+    REAL(KIND=C_DOUBLE), INTENT(OUT)    :: tZero
+    LOGICAL(C_BOOL), INTENT(IN)         :: left_Of_Max
+    LOGICAL(C_BOOL), INTENT(INOUT)      :: error
   
-
-END SUBROUTINE findExactZeros
-
-
-! Define the wrapper subroutine
-SUBROUTINE evaluateImkM_wrapper(x, f, df)
-  ! Evaluate  Im k(t) - m * pi (CDF) - pi/2  or  Im k(t) - m * pi (CDF)
+    REAL(KIND=C_DOUBLE)   :: xacc, fL, fR, dfL, dfR, tstart_update, tMid
+    LOGICAL(C_BOOL)       :: errorHere
   
-  USE Calcs_Imag, ONLY: evaluateImkM  ! Explicitly tell the wrapper where to find the math
-  IMPLICIT NONE
-
-  REAL(KIND=C_DOUBLE), INTENT(IN)   :: x
-  REAL(KIND=C_DOUBLE), INTENT(OUT)  :: f, df
-     
-  CALL evaluateImkM(x, f, df, m_shared) ! Pass the captured 'm' value
-
-END SUBROUTINE evaluateImkM_wrapper
+    ! INITIALIZE ALL LOCAL VARIABLES
+    errorHere = .FALSE.
+    xacc = 1.0E-11_C_DOUBLE
+    fL = 0.0_C_DOUBLE
+    fR = 0.0_C_DOUBLE
+    dfL = 0.0_C_DOUBLE
+    dfR = 0.0_C_DOUBLE
+    tstart_update = 0.0_C_DOUBLE
+    tMid = 0.0_C_DOUBLE
+    tZero = 0.0_C_DOUBLE  ! Initialize OUT parameter too
+    
+    ! Sync the local value of  m  to the shared value.
+    m_shared = m
 
+    ! Set the accuracy
+    xacc = 1.0E-11_C_DOUBLE
   
+    ! Find mmax, which depends on whether we are working with the PDF or the CDF.
+    ! The PDF uses cos Im k(t) in the integrand; the CDF has sin Im k(t) in the integrand.
+    ! Thus, the PDF has integrand zeros at Im k(t) = pi/2 + m * pi/y;
+    !       the CDF has integrand zeros at Im k(t) =        m * pi/y.
+    ! Ensure the bounds actually bound the zero
+  
+    CALL evaluateImkM(tL, fL, dfL, m, errorHere)
+    IF (errorHere) THEN
+      error = .TRUE.
+      RETURN
+    END IF
+
+    CALL evaluateImkM(tR, fR, dfR, m, errorHere)
+    IF (errorHere) THEN
+      error = .TRUE.
+      RETURN
+    END IF
+
+    IF ( (fL * fR) .GT. 0.0_C_DOUBLE ) THEN
+      ! Then bounds do not bound the zero
+       tMid = (tL + tR) / 2.0_C_DOUBLE
+   
+      CALL improveKZeroBounds(m, left_Of_Max, tMid, tL, tR, errorHere)
+      IF (errorHere) THEN
+        IF (Cverbose) CALL DBLEPR("Bounds do not bracket the zero (findExactZeros)", -1, fR, 1)
+        error = .TRUE.
+        RETURN
+      END IF
+    END IF
+
+    ! For robustness, use rtsafe when the  distance between zeros 
+    ! is expected to be small (i.e., in the tail).
+    IF ( m .LE. -3 ) THEN 
+    
+      CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, errorHere)
+      IF (errorHere) THEN
+        error = .TRUE.
+        IF (Cverbose) CALL DBLEPR("ERROR: cannot solve", -1, tZero, 1)
+        RETURN
+      END IF  
+    ELSE IF ( (Cpsmall) .AND. (current_y .LT. current_mu) ) THEN
+    
+      ! When small p and small y, fight harder for good starting bounds
+      CALL improveKZeroBounds(m, left_Of_Max, tStart, tL, tR, error)
+!      IF (error) THEN
+!        IF (Cverbose) CALL DBLEPR("ERROR: cannot solve", -1, tZero, 1)
+!        RETURN
+!      END IF
+      CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, errorHere)
+
+      IF (errorHere) THEN
+        error = .TRUE.
+        IF (Cverbose) CALL DBLEPR("ERROR: cannot solve", -1, tZero, 1)
+        RETURN
+      END IF
+    ELSE
+      ! Default to rtnewton for "easy" cases (e.g., initial zeros)
+      tstart_update = (tL + tR) / 2.0_C_DOUBLE
+  !    CALL rtnewton(i, evaluateImkM_wrapper, tstart_update, xacc, tZero)
+      CALL rtsafe(evaluateImkM_wrapper, tL, tR, xacc, tZero, errorHere)
+      
+      IF (errorHere) THEN
+        error = .TRUE.
+        RETURN
+      END IF
+    END IF
+    
+  
+  END SUBROUTINE findExactZeros
+  
+  
+!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
+  
+  
+  ! Define the wrapper subroutine
+  SUBROUTINE evaluateImkM_wrapper(x, f, df, error)
+    ! Evaluate  Im k(t) - m * pi (CDF) - pi/2  or  Im k(t) - m * pi (CDF)
+    
+    USE Calcs_Imag, ONLY: evaluateImkM  ! Explicitly tell the wrapper where to find the math
+    IMPLICIT NONE
+  
+    REAL(KIND=C_DOUBLE), INTENT(IN)   :: x
+    REAL(KIND=C_DOUBLE), INTENT(OUT)  :: f, df
+    LOGICAL(C_BOOL), INTENT(INOUT)    :: error
+       
+    CALL evaluateImkM(x, f, df, m_shared, error) ! Pass the captured 'm' value
+
+  END SUBROUTINE evaluateImkM_wrapper
+
   
 END MODULE Calcs_K
 
